/*
    Logo Filter for VirtualDub -- Add a bitmap logo to a video 
	with selectable position, alpha blending, and masking.
	Copyright (C) 1999-2006 Donald A. Graft

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

	The author can be contacted at:
	Donald Graft
	neuron2 AT comcast.net.
*/

#include <windows.h>
#include <commctrl.h>
#include <stdio.h>
#include <crtdbg.h>
#include <math.h>
#include <stdlib.h>

#include "ScriptInterpreter.h"
#include "ScriptError.h"
#include "ScriptValue.h"

#include "resource.h"
#include "filter.h"

///////////////////////////////////////////////////////////////////////////

int RunProc(const FilterActivation *fa, const FilterFunctions *ff);
int StartProc(FilterActivation *fa, const FilterFunctions *ff);
int EndProc(FilterActivation *fa, const FilterFunctions *ff);
long ParamProc(FilterActivation *fa, const FilterFunctions *ff);
int InitProc(FilterActivation *fa, const FilterFunctions *ff);
int ConfigProc(FilterActivation *fa, const FilterFunctions *ff, HWND hwnd);
void StringProc(const FilterActivation *fa, const FilterFunctions *ff, char *str);
void ScriptConfig(IScriptInterpreter *isi, void *lpVoid, CScriptValue *argv, int argc);
bool FssProc(FilterActivation *fa, const FilterFunctions *ff, char *buf, int buflen);

///////////////////////////////////////////////////////////////////////////

typedef struct MyFilterData {
	IFilterPreview		*ifp;
	unsigned char		*bitmap, *mask_bitmap, *bits, *mask_bits;
	long				bwidth, bheight, bpitch;
	long				mask_bwidth, mask_bheight, mask_bpitch;
	unsigned int		depth, mask_depth;
	char				filename[1024];
	int					animate;
	int					x, y, alpha;
	int                 transparent, xr, xg, xb, tolerance;
	unsigned int		loadedBMP;
	int					num_logo_frames;
	unsigned long		start;
	unsigned long		fadeinlen, fadeoutend, fadeoutlen;
	unsigned long		duration, loops;
} MyFilterData;

bool FssProc(FilterActivation *fa, const FilterFunctions *ff, char *buf, int buflen)
{
	MyFilterData *mfd = (MyFilterData *)fa->filter_data;
    char save[255];
	int i, j;

	i = j = 0;
	while(mfd->filename[i])
	{
		if (mfd->filename[i] == '\\')
		{
			save[j++] = '\\';
			save[j++] = '\\';
			i++;
		}
		else
		{
			save[j++] = mfd->filename[i++];
		}
	}
	save[j] = 0;
	_snprintf(buf, buflen, "Config(%d, %d, %d, %d, %d, %d, %d, %d, \"%s\", %d, %d, %d, %d, %d, %d, %d)",
		mfd->x,
		mfd->y,
		mfd->alpha,
		mfd->transparent,
		mfd->xr,
		mfd->xg,
		mfd->xb,
		mfd->tolerance,
		save,
		mfd->animate,
		mfd->start,
		mfd->duration,
		mfd->loops,
		mfd->fadeinlen,
		mfd->fadeoutend,
		mfd->fadeoutlen);

	return true;
}

void ScriptConfig(IScriptInterpreter *isi, void *lpVoid, CScriptValue *argv, int argc)
{
	void CountLogoBMPs(MyFilterData *mfd);
	FilterActivation *fa = (FilterActivation *)lpVoid;
	MyFilterData *mfd = (MyFilterData *)fa->filter_data;

	mfd->x				= argv[0].asInt();
	mfd->y				= argv[1].asInt();
	mfd->alpha			= argv[2].asInt();
	mfd->transparent	= argv[3].asInt();
	mfd->xr				= argv[4].asInt();
	mfd->xg				= argv[5].asInt();
	mfd->xb				= argv[6].asInt();
	mfd->tolerance		= argv[7].asInt();
	strcpy(mfd->filename, *argv[8].asString());
	mfd->animate		= !!argv[9].asInt();
	mfd->start			= argv[10].asInt();
	mfd->duration		= argv[11].asInt();
	mfd->loops			= argv[12].asInt();
	mfd->fadeinlen		= argv[13].asInt();
	mfd->fadeoutend		= argv[14].asInt();
	mfd->fadeoutlen     = argv[15].asInt();
	CountLogoBMPs(mfd);
}

ScriptFunctionDef func_defs[]={
	{ (ScriptFunctionPtr)ScriptConfig, "Config", "0iiiiiiiisiiiiiii" },
	{ NULL },
};

CScriptObject script_obj={
	NULL, func_defs
};

struct FilterDefinition filterDef_Logo = {

	NULL, NULL, NULL,		// next, prev, module
	"logo (1.7)",			// name
	"Overlay a bitmap with alpha blending and transparency.",
							// desc
	"Donald Graft", 		// maker
	NULL,					// private_data
	sizeof(MyFilterData),	// inst_data_size

	InitProc,				// initProc
	NULL,					// deinitProc
	RunProc,				// runProc
	ParamProc,				// paramProc
	ConfigProc, 			// configProc
	StringProc, 			// stringProc
	StartProc,				// startProc
	EndProc,				// endProc

	&script_obj,			// script_obj
	FssProc,				// fssProc

};

#define NO_MASK 0
#define TRANSPARENT_COLOR 1
#define ALPHA_MASK 2

// Load a bitmap into memory. BMPno is the index
// of the bitmap in the sequence of bitmaps. If it
// is a static logo, BMPno will always be 0.
int Load(MyFilterData *mfd, unsigned int BMPno, const FilterFunctions *ff)
{
	static FILE *fp, *maskfp;
	char buf[255];
	DWORD size;
    char *p;
	unsigned int found;
	char path[255];

	// Don't load the bitmap if it is already loaded.
	if (BMPno == mfd->loadedBMP)
		return 1;

	strcpy(path, mfd->filename);
	if (mfd->animate)
	{
		p = path + strlen(path);
		while (p >= path && *p != '.') p--;
		found = 0;
		while (*(p-1) >= '0' && *(p-1) <= '9')
		{
			p--;
			found = 1;
		}
		if (found)
		{
			sprintf(p, "%05d.bmp", BMPno);
			if ((fp = fopen(path, "rb")) == NULL)
			{
				ff->Except("Cannot open logo file:\n%s", path);
#if 0
				sprintf(buf, "Cannot open logo file:\n%s", path);
				MessageBox(NULL, buf, "Logo error", MB_OK | MB_TASKMODAL);
#endif
				return 0;
			}
		}
		else
		{
			ff->Except("Malformed animated bitmap file name:\n%s", path);
#if 0
			sprintf(buf, "Malformed animated bitmap file name:\n%s", path);
			MessageBox(NULL, buf, "Logo error", MB_OK | MB_TASKMODAL);
#endif
			return 0;
		}
	}
	else if ((fp = fopen(path, "rb")) == NULL)
	{
		ff->Except("Cannot open logo file:\n%s", path);
#if 0
		sprintf(buf, "Cannot open logo file:\n%s", path);
		MessageBox(NULL, buf, "Logo error", MB_OK | MB_TASKMODAL);
#endif
		return 0;
	}

    // When we reach here, we have the requested bitmap opened.
	mfd->depth = 0;
	// Check that it is a BMP file and get the size.
	fread(buf, 1, 2, fp);
	if (buf[0] != 'B' || buf[1] != 'M')
	{
		ff->Except("File is not a BMP file:\n%s", path);
#if 0
		sprintf(buf, "File is not a BMP file:\n%s", path);
		MessageBox(NULL, buf, "Logo error", MB_OK | MB_TASKMODAL);
#endif
		return 0;
	}
	fread(&size, 1, 4, fp);
	fclose(fp);
	// Now read the BMP into memory and store its parameters.
	if (mfd->bitmap) free(mfd->bitmap);
	mfd->bitmap = (unsigned char *) malloc((size_t) size);
    if ((fp = fopen(path, "rb")) == NULL)
	{
		ff->Except("Could not allocate memory for bitmap:\n%s", path);
#if 0
		sprintf(buf, "Could not allocate memory for bitmap:\n%s", path);
		MessageBox(NULL, buf, "Logo error", MB_OK | MB_TASKMODAL);
#endif
		return 0;
	}
	fread(mfd->bitmap, 1, size, fp);
	fclose(fp);
	mfd->bits = mfd->bitmap + *((DWORD *)(mfd->bitmap + 10));
	mfd->bwidth = *((LONG *)(mfd->bitmap + 18));
	mfd->bheight = *((DWORD *)(mfd->bitmap + 22));
	mfd->bpitch = 3 * mfd->bwidth;
	if (mfd->bpitch & 0x3)
	{
		mfd->bpitch = (mfd->bpitch & 0xfffffffc) + 4;
	}
	mfd->depth = *((unsigned int *)(mfd->bitmap + 28));
	// The depth must be 24 bits.
	if (mfd->depth != 24)
	{
		ff->Except("The logo BMP file depth must be 24 bits.");
#if 0
		MessageBox(NULL, "The logo BMP file depth must be 24 bits.", "Logo error", MB_OK | MB_TASKMODAL);
#endif
		return 0;
	}

	// Load the corresponding mask BMP if we are doing alpha masking.
	if (mfd->transparent == ALPHA_MASK)
	{
		p = path + strlen(path);
		while (p >= path && *p != '.') p--;
		strcpy(p, "A.bmp");
		if ((maskfp = fopen(path, "rb")) == NULL)
		{
			ff->Except("Cannot open mask file:\n%s", path);
#if 0
			sprintf(buf, "Cannot open mask file:\n%s", path);
			MessageBox(NULL, buf, "Logo error", MB_OK | MB_TASKMODAL);
#endif
			return 0;
		}
		mfd->mask_depth = 0;
		// Check that it is a BMP file and get the size.
		fread(buf, 1, 2, maskfp);
		if (buf[0] != 'B' || buf[1] != 'M')
		{
			ff->Except("File is not a BMP file:\n%s", path);
#if 0
			sprintf(buf, "File is not a BMP file:\n%s", path);
			MessageBox(NULL, buf, "Logo error", MB_OK | MB_TASKMODAL);
#endif
			return 0;
		}
		fread(&size, 1, 4, maskfp);
		fclose(maskfp);
		// Now read the BMP into memory and store its parameters.
		if (mfd->mask_bitmap) free(mfd->mask_bitmap);
		mfd->mask_bitmap = (unsigned char *) malloc((size_t) size);
		if ((maskfp = fopen(path, "rb")) == NULL)
		{
			ff->Except("File open problem:\n%s", path);
#if 0
			sprintf(buf, "File open problem:\n%s", path);
			MessageBox(NULL, buf, "Logo error", MB_OK | MB_TASKMODAL);
#endif
			return 0;
		}
		fread(mfd->mask_bitmap, 1, size, maskfp);
		fclose(maskfp);
		mfd->mask_bits = mfd->mask_bitmap + *((DWORD *)(mfd->mask_bitmap + 10));
		mfd->mask_bwidth = *((LONG *)(mfd->mask_bitmap + 18));
		mfd->mask_bheight = *((DWORD *)(mfd->mask_bitmap + 22));
		mfd->mask_bpitch = 3 * mfd->mask_bwidth;
		if (mfd->mask_bpitch & 0x3)
		{
			mfd->mask_bpitch = (mfd->mask_bpitch & 0xfffffffc) + 4;
		}
		mfd->mask_depth = *((unsigned int *)(mfd->mask_bitmap + 28));
		// The depth must be 24 bits.
		if (mfd->mask_depth != 24)
		{
			ff->Except("The mask BMP file depth must be 24 bits.");
#if 0
			MessageBox(NULL, "The mask BMP file depth must be 24 bits.", "Logo error", MB_OK | MB_TASKMODAL);
#endif
			return 0;
		}
		// Verify that the logo and mask BMPs are the same format.
		if (mfd->mask_bwidth != mfd->bwidth ||
			mfd->mask_bheight != mfd->bheight ||
			mfd->mask_depth != mfd->depth)
		{
			ff->Except("The logo and mask BMPs must have the same format.");
#if 0
			MessageBox(NULL, "The logo and mask BMPs must have the same format.", "Logo error", MB_OK | MB_TASKMODAL);
#endif
			return 0;
		}
	}

	// Remember which BMP is loaded.
	mfd->loadedBMP = BMPno;
	return 1;
}

void CountLogoBMPs(MyFilterData *mfd)
{
	char path[255], *p;
	int found, value;
	FILE *fp;

	// Determine the number of BMP files in the logo sequence.
	// RunProc() needs that to calculate whether frames need
	// a bitmap to be overlayed or not. Just open the BMP files
	// in sequence and count them.
	if (mfd->animate)
	{
		strcpy(path, mfd->filename);
		p = path + strlen(path);
		while (p >= path && *p != '.') p--;
		value = 0;
		mfd->num_logo_frames = 0;
		while (1)
		{
			found = 0;
			while (*(p-1) >= '0' && *(p-1) <= '9')
			{
				p--;
				found = 1;
			}
			if (found)
			{
				sprintf(p, "%05d.bmp", value++);
				if ((fp = fopen(path, "rb")) == NULL)
					break;
				else
				{
					mfd->num_logo_frames++;
					fclose(fp);
				}
				p = path + strlen(path);
				while (p >= path && *p != '.') p--;
			}
			else
				break;
		}
	}
	else
		mfd->num_logo_frames = 1;
}

int StartProc(FilterActivation *fa, const FilterFunctions *ff)
{
	MyFilterData *mfd = (MyFilterData *)fa->filter_data;

	// Initialize to a number that will never occur so that the
	// test for whether the bitmap is already loaded in RunProc()
	// will work the first time it is executed.
	mfd->loadedBMP = 0xffffffff;
	return 0;
}

int EndProc(FilterActivation *fa, const FilterFunctions *ff)
{
	MyFilterData *mfd = (MyFilterData *)fa->filter_data;

	if (mfd->bitmap)
	{
		free(mfd->bitmap);
		mfd->bitmap = NULL;
	}
	if (mfd->mask_bitmap)
	{
		free(mfd->mask_bitmap);
		mfd->mask_bitmap = NULL;
	}
	return 0;
}

int RunProc(const FilterActivation *fa, const FilterFunctions *ff) {
	MyFilterData *mfd = (MyFilterData *)fa->filter_data;
	const PixDim	width = fa->src.w;
	const PixDim	height = fa->src.h;
	Pixel32 *src;
	int x, y;
	unsigned long frame;
	int r, g, b, rb, gb, bb;
	unsigned char *bp, *mask_bp;
	int X = mfd->x;
	int Y = mfd->y;
	double ALPHA = mfd->alpha / 255.0;
	double in = mfd->fadeinlen;
	double out = mfd->fadeoutlen;
	int XR = mfd->xr;
	int XG = mfd->xg;
	int XB = mfd->xb;
	bool doit;
	int BMPno;

	// Get the current frame number.
	frame = fa->pfsi->lCurrentFrame;

	// If we are before the starting point, just return
	// the source frame without overlaying a BMP.
	if (frame < mfd->start)
		return 0;

	// If we are after the last frame that should have a BMP
	// overlayed, just return; otherwise determine the index
	// of the BMP to be used.
	if (!mfd->animate)
	{
		// Static logo. If duration is 0, show the logo on all frames.
		if (mfd->duration && frame >= mfd->start + mfd->duration)
			return 0;
		// Always use the first defined file for static logos.
		BMPno = 0;
	}
	else
	{
		// Animated logo. If loops or duration are 0 (infinite), show the logo on all frames.
		if (mfd->duration && mfd->loops && frame >= mfd->start + mfd->loops * mfd->num_logo_frames * mfd->duration)
			return 0;
		// If the per-BMP duration is 0 (infinite), show the first BMP forever.
		if (!mfd->duration)
			BMPno = 0;
		else
		{
			if (mfd->num_logo_frames)
				BMPno = ((frame - mfd->start) / mfd->duration) % mfd->num_logo_frames;
			else
			{
				ff->Except("FAULT: num_logo_frames is zero!");
#if 0
				MessageBox(NULL, "FAULT: num_logo_frames is zero!", "Logo error", MB_OK | MB_TASKMODAL);
#endif
				BMPno = 0;
			}
		}
	}

	// Load the required BMP file.
	if (Load(mfd, BMPno, ff) == 0)
		return 0;

	// Check that BMP isn't larger than the video frame.
	if (mfd->bwidth > width || mfd->bheight > height)
	{
		ff->Except("FAULT: BMP is larger than the video frame!");
//		MessageBox(NULL, "FAULT: BMP is larger than the video frame!", "Logo error", MB_OK | MB_TASKMODAL);
		return 0;
	}
	// Don't try to draw off the frame.
	if (X + mfd->bwidth > width)
		X = width - mfd->bwidth;
	if (Y + mfd->bheight > height)
		Y = height - mfd->bheight;

	// Now render the bitmap onto the current frame.
	src	= (Pixel *)((char *)fa->src.data + (height - Y - mfd->bheight) * fa->src.pitch);
	for (y = 0; y < mfd->bheight; y++)
	{
		bp = mfd->bits + y * mfd->bpitch;
		mask_bp = mfd->mask_bits + y * mfd->mask_bpitch;
		for (x = X; x < X + mfd->bwidth; x++)
		{
			r = ((src[x] & 0xff0000) >> 16);
			g = ((src[x] & 0xff00) >> 8);
			b = (src[x] & 0xff);
			bb = *bp++;
			gb = *bp++;
			rb = *bp++;
			if ((mfd->fadeinlen > 0) && (frame >= mfd->start) && (frame < (mfd->start + in)))
			{
				ALPHA = ((frame - mfd->start) / in) * (mfd->alpha / 255.0);
			}
			if ((mfd->fadeoutlen > 0) && (frame > mfd->fadeoutend - out) /*&&
				(frame <= mfd->fadeoutend)*/)
			{
				if (frame < mfd->fadeoutend)
				{
					ALPHA = ((mfd->fadeoutend - frame) / out) * (mfd->alpha / 255.0);
				}
				else
				{
					ALPHA = 0.0;
				}
			}
			doit = false;
			if (mfd->transparent == NO_MASK)
				doit = true;
			else if ((mfd->transparent == TRANSPARENT_COLOR) && (mfd->tolerance == 0) &&
				     (bb != XB || gb != XG || rb != XR))
				doit = true;
			else if (mfd->transparent == TRANSPARENT_COLOR && mfd->tolerance != 0)
			{
				if (abs(bb - XB) > mfd->tolerance ||
					abs(gb - XG) > mfd->tolerance ||
					abs(rb - XR) > mfd->tolerance)
					doit = true;
			}
			else if (mfd->transparent == ALPHA_MASK)
			{
				double alpha = *mask_bp / 255.0;
				mask_bp += 3;
				bb = (int) (alpha * bb + (1 - alpha) * b);
				gb = (int) (alpha * gb + (1 - alpha) * g);
				rb = (int) (alpha * rb + (1 - alpha) * r);
				src[x] = ((rb << 16) | (gb << 8) | (bb));
				doit = true;
			}
			if (doit == true)
			{
				bb = (int) (ALPHA * bb + (1 - ALPHA) * b);			 
				gb = (int) (ALPHA * gb + (1 - ALPHA) * g);			 
				rb = (int) (ALPHA * rb + (1 - ALPHA) * r);			 
				src[x] = ((rb << 16) | (gb << 8) | (bb));
			}
		}
		src	= (Pixel *)((char *)src + fa->src.pitch);
	}
	return 0;
}

extern "C" int __declspec(dllexport) __cdecl VirtualdubFilterModuleInit2(FilterModule *fm, const FilterFunctions *ff, int& vdfd_ver, int& vdfd_compat);
extern "C" void __declspec(dllexport) __cdecl VirtualdubFilterModuleDeinit(FilterModule *fm, const FilterFunctions *ff);

static FilterDefinition *fd_Logo;

int __declspec(dllexport) __cdecl VirtualdubFilterModuleInit2(FilterModule *fm, const FilterFunctions *ff, int& vdfd_ver, int& vdfd_compat) {
	if (!(fd_Logo = ff->addFilter(fm, &filterDef_Logo, sizeof(FilterDefinition))))
		return 1;

	vdfd_ver = VIRTUALDUB_FILTERDEF_VERSION;
	vdfd_compat = VIRTUALDUB_FILTERDEF_COMPATIBLE;

	return 0;
}

void __declspec(dllexport) __cdecl VirtualdubFilterModuleDeinit(FilterModule *fm, const FilterFunctions *ff) {
	ff->removeFilter(fd_Logo);
}

int InitProc(FilterActivation *fa, const FilterFunctions *ff)
{
	MyFilterData *mfd = (MyFilterData *)fa->filter_data;
   	char prog[256];
	char path[256];
	LPTSTR ptr;

	mfd->x = 0;
	mfd->y = 0;
	mfd->alpha = 255;
	mfd->transparent = 0;
	mfd->animate = 0;
	mfd->xr = 0;
	mfd->xg = 0;
	mfd->xb = 0xff;
	mfd->tolerance = 0;
	mfd->start = 0;
	mfd->fadeinlen = mfd->fadeoutend = mfd->fadeoutlen = 0;
	mfd->duration = 0;
	mfd->loops = 0;

	GetModuleFileName(NULL, prog, 255);
	GetFullPathName(prog, 255, path, &ptr);
	*ptr = 0;
	strcat(path, "plugins\\Logo.bmp");
	strcpy(mfd->filename, path);
	CountLogoBMPs(mfd);
	return 0;
}

BOOL CALLBACK ConfigDlgProc(HWND hdlg, UINT msg, WPARAM wParam, LPARAM lParam) {
	MyFilterData *mfd = (MyFilterData *)GetWindowLong(hdlg, DWL_USER);

	switch(msg) {
		case WM_INITDIALOG:
			SetWindowLong(hdlg, DWL_USER, lParam);
			mfd = (MyFilterData *)lParam;

			if (mfd->transparent == NO_MASK)
			{
				CheckDlgButton(hdlg, IDC_TRANSPARENT_COLOR, BST_UNCHECKED);
				CheckDlgButton(hdlg, IDC_ALPHA_MASK, BST_UNCHECKED);
				EnableWindow(GetDlgItem(hdlg, IDC_RED), FALSE);
				EnableWindow(GetDlgItem(hdlg, IDC_BLUE), FALSE);
				EnableWindow(GetDlgItem(hdlg, IDC_GREEN), FALSE);
				EnableWindow(GetDlgItem(hdlg, IDC_TOLERANCE), FALSE);
			}
			else if (mfd->transparent == TRANSPARENT_COLOR)
			{
				CheckDlgButton(hdlg, IDC_TRANSPARENT_COLOR, BST_CHECKED);
				CheckDlgButton(hdlg, IDC_ALPHA_MASK, BST_UNCHECKED);
				EnableWindow(GetDlgItem(hdlg, IDC_RED), TRUE);
				EnableWindow(GetDlgItem(hdlg, IDC_BLUE), TRUE);
				EnableWindow(GetDlgItem(hdlg, IDC_GREEN), TRUE);
				EnableWindow(GetDlgItem(hdlg, IDC_TOLERANCE), TRUE);
			}
			else if (mfd->transparent == ALPHA_MASK)
			{
				CheckDlgButton(hdlg, IDC_TRANSPARENT_COLOR, BST_UNCHECKED);
				CheckDlgButton(hdlg, IDC_ALPHA_MASK, BST_CHECKED);
				EnableWindow(GetDlgItem(hdlg, IDC_RED), FALSE);
				EnableWindow(GetDlgItem(hdlg, IDC_BLUE), FALSE);
				EnableWindow(GetDlgItem(hdlg, IDC_GREEN), FALSE);
				EnableWindow(GetDlgItem(hdlg, IDC_TOLERANCE), FALSE);
			}

			CheckDlgButton(hdlg, IDC_ANIMATE, mfd->animate ? BST_CHECKED : BST_UNCHECKED);
			SetWindowText(GetDlgItem(hdlg, IDC_FILENAME), mfd->filename);

			SendMessage(GetDlgItem(hdlg, IDC_XSLIDER), TBM_SETRANGE, (WPARAM)TRUE, MAKELONG(0, 1920));
			SendMessage(GetDlgItem(hdlg, IDC_XSLIDER), TBM_SETPOS, (WPARAM)TRUE, mfd->x);
			SetDlgItemInt(hdlg, IDC_XOFF, mfd->x, FALSE);
			SendMessage(GetDlgItem(hdlg, IDC_YSLIDER), TBM_SETRANGE, (WPARAM)TRUE, MAKELONG(0, 1088));
			SendMessage(GetDlgItem(hdlg, IDC_YSLIDER), TBM_SETPOS, (WPARAM)TRUE, mfd->y);
			SetDlgItemInt(hdlg, IDC_YOFF, mfd->y, FALSE);
			
			SetDlgItemInt(hdlg, IDC_RED, mfd->xr, FALSE);
			SetDlgItemInt(hdlg, IDC_ALPHA, mfd->alpha, FALSE);
			SetDlgItemInt(hdlg, IDC_GREEN, mfd->xg, FALSE);
			SetDlgItemInt(hdlg, IDC_BLUE, mfd->xb, FALSE);
			SetDlgItemInt(hdlg, IDC_TOLERANCE, mfd->tolerance, FALSE);
			SetDlgItemInt(hdlg, IDC_START, mfd->start, FALSE);
			SetDlgItemInt(hdlg, IDC_DURATION, mfd->duration, FALSE);
			SetDlgItemInt(hdlg, IDC_LOOPS, mfd->loops, FALSE);
			SetDlgItemInt(hdlg, IDC_FADEINLEN, mfd->fadeinlen, FALSE);
			SetDlgItemInt(hdlg, IDC_FADEOUTEND, mfd->fadeoutend, FALSE);
			SetDlgItemInt(hdlg, IDC_FADEOUTLEN, mfd->fadeoutlen, FALSE);
			mfd->ifp->InitButton(GetDlgItem(hdlg, IDPREVIEW));
			return TRUE;

		case WM_HSCROLL:
			if ((HWND) lParam == GetDlgItem(hdlg, IDC_XSLIDER))
			{
				int x = SendMessage(GetDlgItem(hdlg, IDC_XSLIDER), TBM_GETPOS, 0, 0);
				if (x != mfd->x)
				{
					mfd->x = x;
					SetDlgItemInt(hdlg, IDC_XOFF, mfd->x, FALSE);
					mfd->ifp->RedoFrame();
				}
			}
		case WM_VSCROLL:
			if ((HWND) lParam == GetDlgItem(hdlg, IDC_YSLIDER))
			{
				int y = SendMessage(GetDlgItem(hdlg, IDC_YSLIDER), TBM_GETPOS, 0, 0);
				if (y != mfd->y)
				{
					mfd->y = y;
					SetDlgItemInt(hdlg, IDC_YOFF, mfd->y, FALSE);
					mfd->ifp->RedoFrame();
				}
			}
			break;

		case WM_COMMAND:
			switch(LOWORD(wParam)) {

			case IDPREVIEW:
				mfd->ifp->Toggle(hdlg);
				break;

			case IDOK:
				EndDialog(hdlg, 0);
				return TRUE;
			case IDHELP:
				{
				char prog[256];
				char path[256];
				LPTSTR ptr;

				GetModuleFileName(NULL, prog, 255);
				GetFullPathName(prog, 255, path, &ptr);
				*ptr = 0;
				strcat(path, "plugins\\Logo.html");
				ShellExecute(hdlg, "open", path, NULL, NULL, SW_SHOWNORMAL);
				return TRUE;
				}

			case IDC_BROWSE:
				{
				OPENFILENAME ofn;
				mfd->filename[0] = NULL;
				ofn.lStructSize = sizeof(OPENFILENAME);
				ofn.hwndOwner = hdlg;
				ofn.hInstance = NULL;
				ofn.lpTemplateName = NULL;
				ofn.lpstrFilter = "All Files\0*.*\0\0";
				ofn.lpstrCustomFilter = NULL;
				ofn.nMaxCustFilter = 0;
				ofn.nFilterIndex = 1;
				ofn.lpstrFile = mfd->filename;
				ofn.nMaxFile = 1024;
				ofn.lpstrFileTitle = NULL;
				ofn.nMaxFileTitle = 0;
				ofn.lpstrInitialDir = NULL;
				ofn.lpstrTitle = "Choose Logo Bitmap File";
				ofn.Flags = OFN_CREATEPROMPT;
				ofn.nFileOffset = 0;
				ofn.nFileExtension = 0;
				ofn.lpstrDefExt = NULL;
				ofn.lCustData = 0;
				ofn.lpfnHook = NULL;
				GetOpenFileName(&ofn);
				if (mfd->filename[0] != 0)
				{
					SetWindowText(GetDlgItem(hdlg, IDC_FILENAME), mfd->filename);
					CountLogoBMPs(mfd);
					mfd->ifp->UndoSystem();
					mfd->ifp->RedoSystem();
				}
				break;
				}

			case IDC_ALPHA:
				mfd->alpha = GetDlgItemInt(hdlg, IDC_ALPHA, 0, FALSE);
				if (mfd->alpha > 255)
				{
					SetDlgItemInt(hdlg, IDC_ALPHA, 255, FALSE);
					mfd->alpha = 255;
				}
				mfd->ifp->RedoFrame();
				break;
			case IDC_ALPHA_MASK:
				if (IsDlgButtonChecked(hdlg, IDC_ALPHA_MASK))
				{
					CheckDlgButton(hdlg, IDC_TRANSPARENT_COLOR, BST_UNCHECKED);
					mfd->transparent = ALPHA_MASK;
				}
				else
				{
					CheckDlgButton(hdlg, IDC_TRANSPARENT_COLOR, BST_UNCHECKED);
					mfd->transparent = NO_MASK;
				}
				EnableWindow(GetDlgItem(hdlg, IDC_RED), FALSE);
				EnableWindow(GetDlgItem(hdlg, IDC_BLUE), FALSE);
				EnableWindow(GetDlgItem(hdlg, IDC_GREEN), FALSE);
				EnableWindow(GetDlgItem(hdlg, IDC_TOLERANCE), FALSE);
				mfd->ifp->UndoSystem();
				mfd->ifp->RedoSystem();
				break;
			case IDC_TRANSPARENT_COLOR:
				if (IsDlgButtonChecked(hdlg, IDC_TRANSPARENT_COLOR))
				{
					CheckDlgButton(hdlg, IDC_ALPHA_MASK, BST_UNCHECKED);
					mfd->transparent = TRANSPARENT_COLOR;
					EnableWindow(GetDlgItem(hdlg, IDC_RED), TRUE);
					EnableWindow(GetDlgItem(hdlg, IDC_BLUE), TRUE);
					EnableWindow(GetDlgItem(hdlg, IDC_GREEN), TRUE);
					EnableWindow(GetDlgItem(hdlg, IDC_TOLERANCE), TRUE);
				}
				else
				{
					CheckDlgButton(hdlg, IDC_ALPHA_MASK, BST_UNCHECKED);
					mfd->transparent = NO_MASK;
					EnableWindow(GetDlgItem(hdlg, IDC_RED), FALSE);
					EnableWindow(GetDlgItem(hdlg, IDC_BLUE), FALSE);
					EnableWindow(GetDlgItem(hdlg, IDC_GREEN), FALSE);
					EnableWindow(GetDlgItem(hdlg, IDC_TOLERANCE), FALSE);
				}
				mfd->ifp->UndoSystem();
				mfd->ifp->RedoSystem();
				break;
			case IDC_ANIMATE:
				mfd->animate = !!IsDlgButtonChecked(hdlg, IDC_ANIMATE);
				CountLogoBMPs(mfd);
				mfd->ifp->UndoSystem();
				mfd->ifp->RedoSystem();
				break;
			case IDC_RED:
				mfd->xr = GetDlgItemInt(hdlg, IDC_RED, 0, FALSE);
				if (mfd->xr > 255)
				{
					SetDlgItemInt(hdlg, IDC_RED, 255, FALSE);
					mfd->xr = 255;
				}
				mfd->ifp->RedoFrame();
				break;
			case IDC_GREEN:
				mfd->xg = GetDlgItemInt(hdlg, IDC_GREEN, 0, FALSE);
				if (mfd->xg > 255)
				{
					SetDlgItemInt(hdlg, IDC_GREEN, 255, FALSE);
					mfd->xg = 255;
				}
				mfd->ifp->RedoFrame();
				break;
			case IDC_BLUE:
				mfd->xb = GetDlgItemInt(hdlg, IDC_BLUE, 0, FALSE);
				if (mfd->xb > 255)
				{
					SetDlgItemInt(hdlg, IDC_BLUE, 255, FALSE);
					mfd->xb = 255;
				}
				mfd->ifp->RedoFrame();
				break;
			case IDC_TOLERANCE:
				mfd->tolerance = GetDlgItemInt(hdlg, IDC_TOLERANCE, 0, FALSE);
				if (mfd->tolerance > 255)
				{
					SetDlgItemInt(hdlg, IDC_TOLERANCE, 255, FALSE);
					mfd->tolerance = 255;
				}
				mfd->ifp->RedoFrame();
				break;
			case IDC_START:
				mfd->start = GetDlgItemInt(hdlg, IDC_START, 0, FALSE);
				mfd->ifp->RedoFrame();
				break;
			case IDC_FADEINLEN:
				mfd->fadeinlen = GetDlgItemInt(hdlg, IDC_FADEINLEN, 0, FALSE);
				mfd->ifp->RedoFrame();
				break;
			case IDC_FADEOUTEND:
				mfd->fadeoutend = GetDlgItemInt(hdlg, IDC_FADEOUTEND, 0, FALSE);
				mfd->ifp->RedoFrame();
				break;
			case IDC_FADEOUTLEN:
				mfd->fadeoutlen = GetDlgItemInt(hdlg, IDC_FADEOUTLEN, 0, FALSE);
				mfd->ifp->RedoFrame();
				break;
			case IDC_DURATION:
				mfd->duration = GetDlgItemInt(hdlg, IDC_DURATION, 0, FALSE);
				mfd->ifp->RedoFrame();
				break;
			case IDC_LOOPS:
				mfd->loops = GetDlgItemInt(hdlg, IDC_LOOPS, 0, FALSE);
				mfd->ifp->RedoFrame();
				break;
			case IDC_XPLUS:
				if (mfd->x < 1200)
				{
					mfd->x += 1;
					SetDlgItemInt(hdlg, IDC_XOFF, mfd->x, FALSE);
					SendMessage(GetDlgItem(hdlg, IDC_XSLIDER), TBM_SETPOS, (WPARAM)TRUE, mfd->x);
					mfd->ifp->RedoFrame();
				}
				break;
			case IDC_XMINUS:
				if (mfd->x > 0)
				{
					mfd->x -= 1;
					SetDlgItemInt(hdlg, IDC_XOFF, mfd->x, FALSE);
					SendMessage(GetDlgItem(hdlg, IDC_XSLIDER), TBM_SETPOS, (WPARAM)TRUE, mfd->x);
					mfd->ifp->RedoFrame();
				}
				break;
			case IDC_YPLUS:
				if (mfd->y < 800)
				{
					mfd->y += 1;
					SetDlgItemInt(hdlg, IDC_YOFF, mfd->y, FALSE);
					SendMessage(GetDlgItem(hdlg, IDC_YSLIDER), TBM_SETPOS, (WPARAM)TRUE, mfd->y);
					mfd->ifp->RedoFrame();
				}
				break;
			case IDC_YMINUS:
				if (mfd->y > 0)
				{
					mfd->y -= 1;
					SetDlgItemInt(hdlg, IDC_YOFF, mfd->y, FALSE);
					SendMessage(GetDlgItem(hdlg, IDC_YSLIDER), TBM_SETPOS, (WPARAM)TRUE, mfd->y);
					mfd->ifp->RedoFrame();
				}
				break;
			case IDCANCEL:
				EndDialog(hdlg, 1);
				return TRUE;
			}
			break;
	}

	return FALSE;
}

int ConfigProc(FilterActivation *fa, const FilterFunctions *ff, HWND hwnd)
{
	MyFilterData *mfd = (MyFilterData *)fa->filter_data;
	MyFilterData mfd_old = *mfd;
	int ret;

	mfd->ifp = fa->ifp;

	if (DialogBoxParam(fa->filter->module->hInstModule, MAKEINTRESOURCE(IDD_FILTER),
		hwnd, ConfigDlgProc, (LPARAM)mfd))
	{
		*mfd = mfd_old;
		ret = TRUE;
	}
    else
	{
		ret = FALSE;
	}
	return(ret);
}

void StringProc(const FilterActivation *fa, const FilterFunctions *ff, char *str)
{
	MyFilterData *mfd = (MyFilterData *)fa->filter_data;
}

long ParamProc(FilterActivation *fa, const FilterFunctions *ff)
{
	fa->dst.offset	= fa->src.offset;
	fa->dst.modulo	= fa->src.modulo;
	fa->dst.pitch	= fa->src.pitch;
	return 0;
}
